#include <iostream>


/*

  Implementation de RANDU, un mauvais generateur de nombres aleatoire, defini par
  x_{n+1} = (2^16 + 2 + 1) * x_n  modulo  2^31

  Ce generateur de periode 2^29 a ete utilise dans les annees 60-70, mais ses valeurs
  sont beaucoup trop corellees : les triplets (x_n,x_{n+1},x_{n+2}) ne remplissent pas du tout l'espace {1,...,2^31-1}^3

*/


unsigned N = 0x80000000U; // 2^31 = 8x2^28 = 8x16^7, en base 16. Le "U" force l'interpretation de la valeur comme "unsigned"


int randu(void)
{
  static int x = 1; // Graine du generateur : on pourrait partir de n'importe quelle autre valeur. La donnee de la graine determine la liste de nombre obtenue.

  // La variable est declaree statique de sorte a garder en memoire l'etat du generateur.

  x = ( 65539 * x ) % N;

  return x;
}



int main(void)
{
  for (int i=0; i<20;i++)
	std::cout << randu() << "\t" << randu()/(double)N << "\n";

  /*
	randu() renvoi des valeurs dans {1,...,N-1}, alors que
	randu()/(double)N renvoie des valeurs dans ]0,1[.
  */
  
  return 0;

}
