#include <iostream>
#include <cmath>

/*

  Illustration de problèmes rencontrés avec le générateur RANDU pour
  un algorithme de Monte Carlo

*/



// La fonction randu renvoie une double uniforme sur [0,1]
unsigned N = 0x80000000U;
double randu(void)
{
  static int x = 1; // La graine du générateur
  x = ( 65539 * x ) % N;
  return x/(double)N;
}


// La fonction F renvoie 1 si un point tiré au hasard dans [0,1]^3
// tombe dans un petit cube fixé de dimensions 0.1 x 0.1 x 0.1,
// et renvoie 0 sinon.
int F()
{
  double x,y,z;
  x = randu();
  y = randu();
  z = randu();
  if ((0.4<x && x<0.5) && (0.4<y && y<0.5) && (0.5<z && z<0.6))
	return 1;
  else
	return 0;
}




int main(void)
{
  int k = 10;
  int somme = 0;
  double v = 0.1*0.1*0.1; //Volume du cube de la fonction F
  
  std::cout << "Estimation du volume d'un petit cube, par la méthode de Monte Carlo utilisant l'algorithme RANDU.\n";

  for (int n=0 ; n<=10000000 ; n++)
	{
	  // On compte le nombre de fois que l'on atteint le petit cube
	  somme += F();
	  
	  if (n==k) // On affiche le résultat au bout de 10, 100, 1000... simulations
		{
		  k *= 10;

		  std::cout << "Au bout de " << n << " simulations :\n";
		  std::cout << "Nombre de points tombés dans le cube : " << somme << "\n";
		  std::cout << "En théorie, la valeur devrait être dans [ " << n*v - 1.96*sqrt(n*v*(1-v)) << " ; " << n*v + 1.96*sqrt(n*v*(1-v)) << "  ] (avec probabilité 95%)\n\n";
		}
  
		    

	}
  
  return 0;

}
